#include <Modelib.h>
#include <fstream>
#include <sstream>
using namespace std;
using namespace Modelib;

enum DIR_Cocyle{
	INTERIEUR=0,
	EXTERIEUR=1
};

#define COCYCLE(n,o) Omega<n,2,unsigned>(o)


int main(int argc, char *argv[])
{
	cout << "Creation des donnes" << endl;

	/*Cration des arcs*/
	Ensemble< Nuplet<2,unsigned> > arcs;
	ifstream file("topo.dat");
	unsigned nbsommets = 0;
	unsigned nbarcs = 0;
	file >> nbsommets;
	file >> nbarcs;
	for(unsigned i=0;i<nbarcs;++i)
	{
		file >> arcs;
	}

	//On duplique les arcs parce que le pb est non orient
	for(unsigned i=0;i<nbarcs;++i)
	{
		Nuplet<2,unsigned> tmp = arcs.Get(i);
		unsigned swap = tmp[0];
		tmp[0] = tmp[1];
		tmp[1] = swap;
		arcs.Add( tmp );
	}
	file.close();
	cout << "Arcs:" << endl << arcs << endl;

	/*Cration des commodits*/

	Ensemble< Nuplet<3,unsigned> > commo;
	file.open("commo.dat");
	unsigned nbcommo = 0;
	file >> nbcommo;
	for(unsigned i=0;i<nbcommo;++i)
	{
		file >> commo;
	}
	file.close();
	cout << "Commodit:" << endl << commo << endl;

	/*Cration des equipement*/
	Ensemble< Nuplet<2,unsigned> > equip;
	file.open("equip.dat");
	unsigned nbequip = 0;
	file >> nbequip;
	for(unsigned i=0;i<nbequip;++i)
	{
		file >> equip;
	}
	file.close();
	cout << "Equipement:" << endl << equip << endl;

	cout << "Cration du PL" << endl;

	Model flot;

	flot.SetProblemName("Probleme du flot maximal sur un graphe");

	/* Definition des variables */

	vector< NumVarFamily<2,unsigned>* > y;
	for(unsigned i=0;i<nbcommo;++i)
	{
		ostringstream nom;
		nom << "y%%_" << i;
		y.push_back(new FloatVarFamily<2,unsigned>(flot,0.0f,Infinity,arcs,nom.str()));
	}

	vector< NumVarFamily<2,unsigned>* > x;
	for(unsigned i=0;i<nbequip;++i)
	{
		ostringstream nom;
		nom << "x%%_" << i;
		x.push_back(new BoolVarFamily<2,unsigned>(flot,arcs,nom.str()));
	}

	/*Definition de la fonction objectif*/
	Expr obj;
	for(unsigned i=0;i<nbequip;++i)
	{
		NumVarArray t = x[i]->GetFamilyPart( arcs.all() );
		obj+= 0.5*(equip.Get(i))[1]*Sum(t);
	}

	flot.Minimize(obj,"obj");

	/*Definition des contraintes*/
	for(unsigned i=0;i<nbcommo;++i)
	{
		unsigned o = (commo.Get(i))[0];
		unsigned q = (commo.Get(i))[2];

		NumVarArray t = y[i]->GetFamilyPart( COCYCLE(INTERIEUR,o) );
		flot.Add( -1.0*Sum(t) ==  -q, Nom("c1a_",i));

		NumVarArray u = y[i]->GetFamilyPart( COCYCLE(EXTERIEUR,o) );
		flot.Add( -1.0*Sum(u) ==  0, Nom("c2a_",i));
	}

	for(unsigned i=0;i<nbcommo;++i)
	{
		unsigned o = (commo.Get(i))[0];
		unsigned d = (commo.Get(i))[1];

		for(unsigned j=0;j<nbsommets;++j)
		{
			if(j!=o && j!=d)
			{
				NumVarArray t = y[i]->GetFamilyPart(COCYCLE(INTERIEUR,j));
				NumVarArray u = y[i]->GetFamilyPart(COCYCLE(EXTERIEUR,j));
				flot.Add( Sum(t) - Sum(u) == 0, Nom("c1b_",i,"_",j) );
			}
		}
	}

	for(unsigned i=0;i<nbcommo;++i)
	{
		unsigned d = (commo.Get(i))[1];
		unsigned q = (commo.Get(i))[2];
		ostringstream nom;
		nom << "c1c_" << i;

		NumVarArray t = y[i]->GetFamilyPart( COCYCLE(EXTERIEUR,d) );
		flot.Add( Sum(t) == q, nom.str() );

		NumVarArray u = y[i]->GetFamilyPart( COCYCLE(INTERIEUR,d) );
		flot.Add( Sum(u) == 0, Nom("c2c_",i));
	}


	for(unsigned i=0;i<nbarcs*2;++i)
	{
		Expr tmp;

		for(unsigned j=0;j<nbequip;++j)
		{
			tmp += x[j]->Get(i);
		}
		flot.Add( tmp <= 1, Nom("c2a_",i));
	}

	for(unsigned i=0;i<nbequip;++i)
	{
		for(unsigned j = 0; j < nbarcs; ++j)
		{
		flot.Add( x[i]->Get(j) - x[i]->Get(j+nbarcs) == 0, Nom( "c2b_",i, "_",j));
		}
	}

	for(unsigned i =0; i<nbarcs*2 ;++i)
	{
		Expr tmp1;
		for(unsigned j =0; j < nbcommo; ++j)
		{
			tmp1+=y[j]->Get(i);
		}

		Expr tmp2;
		for(unsigned j=0; j < nbequip; ++j)
		{
			tmp2+=x[j]->Get(i)*(equip.Get(j))[0];
		}

		flot.Add( tmp1 <= tmp2 , Nom("c3_",i) );
	}

	cout << "Resolution" << endl;
	flot.InitSolver(CPLEX, "flotmax.lp");
	cout << "Solution: " << flot.Solve() << endl;
	flot.SortieHTML("flotmax.html");

	flot.OutDimensions();

	for(unsigned i=0;i<nbequip;++i)
		delete x[i],
		delete y[i];

	return 0;
}
