/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __EXPRNODE_H_
#define __EXPRNODE_H_

#include "Config.h"
#include "Num.h"
#include "NumVar.h"
#include "LagrangianParam.h"

namespace Modelib {

//class LagrangianParam;
///Classe representant les briques elementaires des expressions lineaires.
/*
  Elle est constitue
  - d'un coefficient de type "Num"
  - d'une variable de type "NumVar"
  - d'un coefficent Lagrangien
  \n Elle represente un groupe  "+ Num * NumVar" dans une expressions linaire
*/

class ExprNode
{
  protected:
    Num coefficient; //!< Coefficient numerique
    NumVar variable; //!< Variable
    LagrangianParam mult; //!< Variable quadratique facultative
  public:
    /**Constructeur par dfaut et de l'utilisateur*/
    ExprNode(const Num & _coefficient = Num(0.0f),
             const NumVar & _variable = NumVar(),
             const LagrangianParam & _mult = LagrangianParam())
    : coefficient(_coefficient), variable(_variable), mult(_mult)
    {}
 
    /**Seter de la constante
      @return le coefficient du bloc
     */
    Num       GetCoefficient()  const { return coefficient; }
    /**Seter de la constante
      @param le coefficient du bloc
     */
    void SetCoefficient( float coef ) { coefficient = Num(coef); }
    
    /**Geter de la variable
      @return le variable du bloc
     */
    NumVar    GetVariable()     const { return variable;    }
    
    /**Geter du multiplicateur de Lagrange
      @return le multiplicateur de Lagrange du bloc
     */
    LagrangianParam   GetLagrangianParam()    const { return mult;   }   
}; 

}

#endif
