/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef __FICHIER_PROBLEME_H
#define __FICHIER_PROBLEME_H

#include <string>
#include <list>
#include <vector>
#include <map>
#include <iostream>
#include <algorithm>

namespace Modelib {

// type de comparaison
enum CompType {LOWER_OR_EQUAL_THAN=0,LOWER_THAN=1,EQUAL=2,UPPER_OR_EQUAL_THAN=3,UPPER_THAN=4,DIFFERENT_THAN=5};
// signes
typedef enum {MINUS=0,PLUS} Signe;
// outils de comparaisons
static const std::string comp[6] = {
  "<=",
  "<",
  ">=",
  ">",
  "!=", // on vera si on utilise ca avec les expression
  //"/=",  // a!=b =)> a<b ou a>b
  "="   // = a la fin car on test d'abord les spcialisation des cas (composs)
};
// Les diffrentes parties d'un fichier LP
enum {MAXIMIZE=0,SUBJECT,CONSTRAINT,BOUNDS,END,GENERALS,BINARIES,MINIMIZE,FREE};


/** Un monme est dcompos en Signe * Num * Var
    ex: - 2 * X1
*/
struct plExprNode
{
  std::string var;
  std::string num;
  Signe       signe;

  /** Constructeur passant chaque lment
  */
  plExprNode(const std::string& _var, const std::string& _num, const Signe _signe)
    : var(_var), num(_num), signe(_signe)
  {};

  /** Constructeur de copie
      @params _node le monome source
  */
  plExprNode(const plExprNode& _node)
    : var(_node.var), num(_node.num), signe(_node.signe)
  {};
};

/** Dfiniction d'une contrainte de type:
    minBound comparateur1 num * var comparateur2 maxBound
*/
struct plConstraintNode
{
  std::string minBound,maxBound;
  std::string num;
  std::string var;
  unsigned    comp1,comp2;

  plConstraintNode(const std::string& _var, const std::string& _num,
                   const std::string& _b1 , const std::string& _b2 ,
                   const unsigned     _c1 , const unsigned     _c2)
    : minBound(_b1), maxBound(_b2), num(_num), var(_var), comp1(_c1), comp2(_c2)
  {};
  
  /** Constructeur de copie
      @params _node le monome source
  */
  plConstraintNode(const plConstraintNode& _node)
    : minBound(_node.minBound), maxBound(_node.maxBound), num(_node.num), var(_node.var), comp1(_node.comp1), comp2(_node.comp2)
  {};


  
  /*

    faire des constructeurs pour les formes simples
    x <= 0                   _num _var _comp1 _maxbound
    0 <= x                   _num _var _minbound _comp1

  */

  /** Constructeur de comparateur partiel pour une expression de type
      x <= 0                   _num _var _comp1 _maxbound
  */
  plConstraintNode(const std::string& _num, const std::string& _var,
                   const unsigned     _c1 , const std::string& _maxbound)
    : minBound("-inf"), maxBound(_maxbound), num(_num), var(_var), comp1(_c1), comp2(99)
  {};

  /** Constructeur de comparateur partiel pour une expression de type
      0 <= x                   _num _var _minbound _comp1
  */
  plConstraintNode(const std::string& _num, const std::string& _var,
                   const std::string& _minbound, const unsigned _c1 )
    : minBound(_minbound), maxBound("+inf"), num(_num), var(_var), comp1(_c1), comp2(99)
  {};


};

/** Reprsentation d'un maillon d'une expression qui sera le test
    comme "<= 42" etc.
*/
struct plSimpleConstraint
{
  std::string bound;
  unsigned    comp ;

  plSimpleConstraint(const std::string& _bound, const unsigned _comp)
    : bound(_bound), comp(_comp)
  {};

  plSimpleConstraint(const plSimpleConstraint& _cstr)
    : bound(_cstr.bound), comp(_cstr.comp)
  {};

  plSimpleConstraint& operator=(const plSimpleConstraint& _cstr)
  {
    comp  = _cstr.comp;
    bound = _cstr.bound;
    return *this;
  }
};


/** Une plGlobalNode est un noeud qui est dcrit par:
    un nom
    une expression
    un comparateur
    
    ex: 
     contrainte_1 : 1*X1 - 2*X2 + 3*X42 > 0
*/
struct plGlobalNode
{
  std::string              nom ;
  std::list<plExprNode *>  expr;
  plSimpleConstraint       cont;

  plGlobalNode(const plGlobalNode& _node)
    : nom(_node.nom), cont(_node.cont)
  {
    //copy(_node.expr.begin(), _node.expr.end(),
    //           expr.begin());
    for (std::list<plExprNode *>::const_iterator _iter  = _node.expr.begin();
                                                 _iter != _node.expr.end()  ;
                                               ++_iter)
      expr.push_back(*_iter);
  }
  
  plGlobalNode(const std::string& _nom, const std::list<plExprNode *>& _expr, const plSimpleConstraint& _cont)
    : nom(_nom), cont(_cont)
  {
    //copy(_expr.begin(), _expr.end(),
    //      expr.begin());
    for (std::list<plExprNode *>::const_iterator _iter  = _expr.begin();
                                                 _iter != _expr.end()  ;
                                               ++_iter)
      expr.push_back(*_iter);
  }
  
  /** Destructeur, on parcourt toute la liste d'expression pour
      effacer chaque noeud point
  */
  ~plGlobalNode()
  {
    for (std::list<plExprNode*>::iterator i=expr.begin();i!=expr.end();++i)
        delete (*i);
  }
};


/** Functor de comparaison pour les strings
    C'est utilis dans le map
    
    Bon, je trouve que le functor est smentiquement crade, mais bon
    est-ce qu'on y peut quelque chose ? P- qu'en fait, c'est naturel
    de traduire l'instanciation d'une fonction par un objet...
*/
struct Cmp
{
  bool operator()(std::string& s1, std::string& s2) const
  {
    // tout ca, pour ca !
  	return s1 <= s2;
  }
};

class Model;

/** FichierProbleme est une super classe 
    Elle contient tout :]
    
    Ainsi, il est facile d'ajouter des supports de nouveaux types 
    de fichier (je pense notamment au MPS)
    
    Dans tous les cas, la structure devra etre la meme car notre
    'Model' est base dessus (structure grandement equivalente)
*/
class FichierProbleme
{
protected:
  /** Des fois, ca pourrait etre sympa que le probleme ait un nom
  */
  std::string nom;
  /** Est-ce que l'on va maximiser ou minimiser l'expression ?
  */
  bool isMax; // si non alors c Minimize
  /** fonction objectif
  */
  std::list<plExprNode *> exprObj;
  /** les contraintes sont de la forme "NomContrainte: expression contrainte(<=0...)"
  */
  std::list<plGlobalNode *> exprConstraint;
  /** variables entieres
  */
  std::list<std::string> exprGeneral;
  /** variables boolenes
  */
  std::list<std::string> exprBinary;
  /** Borner les variables
  */
  std::list<plConstraintNode *> exprBounds;

  /** Typiquement, on utilisera ce multimap pour le switch
      Comme en C++ on ne peut pas switcher sur des strings, on utilise
      ca pour que ce soit assez rapide.
  */
  std::multimap<std::string,unsigned>      mmap;

  //! Obtenir les comparateurs
  std::map<std::string,unsigned>       Comp2Int;

  /** Chopper les commentaires
  */
  std::list< std::pair<long,std::string> > commentaires;

protected:
  //! Constructeur par defaut
  FichierProbleme() {};

public:
  //! Ouvrir un fichier (nom) et le charger.
  virtual bool Open(const std::string& nom) = 0;

  friend std::ostream& operator<<(std::ostream&,const FichierProbleme&);
  bool operator>> (Model & m);

  /** Notre destructeur
  */
  virtual ~FichierProbleme()
  {
    //std::cout << "Destructeur: ~FichierProbleme()"  << std::endl;
    for (std::list<plExprNode*>::iterator i=exprObj.begin();i!=exprObj.end();++i)
        delete (*i);
    for (std::list<plGlobalNode*>::iterator i=exprConstraint.begin();i!=exprConstraint.end();++i)
        delete (*i);
    for (std::list<plConstraintNode*>::iterator i=exprBounds.begin();i!=exprBounds.end();++i)
        delete (*i);
    // le reste est auto-dtruit grace aux auto_ptr<>
  };
};


}

#endif
