/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __FRACTION_H
#define __FRACTION_H

#include <map>

namespace Modelib {

/**
	Cette classe permet de transformer a la volee 
	des nombres en une fraction les representant.
*/
class FractionConvert
{
  //! Conteneur de notre fraction
  struct Fraction
  {
    unsigned num;	//!< Numerateur
    unsigned den;	//!< Denominateur

	//! Constructeur par defaut
    Fraction(unsigned _num = 0, unsigned _den = 1)
    : num(_num), den(_den)
    {}
    
    //! Constructeur par copie
    Fraction(const Fraction & f)
    : num(f.num), den(f.den)
    {}

	//! Operateur de copie
    Fraction & operator= (const Fraction & f)
    {
      if( &f != this)
      {
        num = f.num;
        den = f.den;
      }
      return *this;
    }
  };

  Fraction * frac;	//!< Pointeur sur un objet de type Fraction
  float    * reel;	//!< Pointeur sur un nombre flottant
  unsigned  nfrac;	//!< nfrac est le nombre de fractions possible avec le degre de precision 'n'

public:
   //! Constructeur par defaut
   FractionConvert(unsigned _n = 100);
   //! Destructeur
  ~FractionConvert();
   //! Approximation d'un nombre par une fraction
   bool Approx(float nb, int & num, unsigned  & den) const;
};

}

#endif

