/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __INTERNALCONSTRAINT_H
#define __INTERNALCONSTRAINT_H

namespace Modelib {

///Structure d'identifiant pour contrainte
struct InternalConstraint
{
    enum Type {INF,SUP,EGAL,INFSTRICT,SUPSTRICT};
    Type type;
    float b;
    float activity;
    float lagrangianValue;
    std::string name;
    bool isLagrangianParamDefined;
    bool isLagrangianParamActive;


    /**Constructeur par dfaut*/
    InternalConstraint(InternalConstraint::Type _type = InternalConstraint::INF,
                       float _b = 0.0f,
                       float _activity = 0.0f,
                       float _lagrangianValue = 0.0f,
                       std::string _name = "",
                       bool _isLagrangianParamDefined = false,
                       bool _isLagrangianParamActive = false)
    : type(_type),
      b(_b),
      activity(_activity),
      lagrangianValue(_lagrangianValue),
      name(_name),
      isLagrangianParamDefined(_isLagrangianParamDefined),
      isLagrangianParamActive(_isLagrangianParamActive)
     {}
    /**
      Constructeur spcifique  partir d'un type et d'un numero de contrainte
      @param  _type va represent le type de contrainte
      @param _numero va represente le numero de la contrainte  savoir le numero de
      la ligne correspondant  la contrainte dans la matrice creuse
     */
    /**Constructeur par recopie*/
    InternalConstraint(const InternalConstraint &_iC)
    {
    	*this =_iC;
    }
    /**Oprateur de recopie*/
    InternalConstraint & operator=(const InternalConstraint &_iC)
    {
       if(this != &_iC)
       {
          type=_iC.type;
          b=_iC.b;
          activity =_iC.activity;
          lagrangianValue =_iC.lagrangianValue;
          name =_iC.name;
          isLagrangianParamDefined =_iC.isLagrangianParamDefined;
          isLagrangianParamActive =_iC.isLagrangianParamActive;
       }
       return *this;
    }
    ~InternalConstraint() {}
};

}

#endif

