/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __MULTLAGRANGE_H
#define __MULTLAGRANGE_H

#include "Config.h"
#include "Constant.h"

namespace Modelib {

class Model; //vite une dclaration cyclique

//!Classe reprsentant des multiplicateurs de Lagrange rattachs  un modle

class LagrangianParam
{
protected:
    VarId id; //!< identifiant de variable
    Model * model; //!< modle de dfinition
    
#ifdef COMPILE_FOR_PYTHON
public:
  // On redefini nos surcharges d'oprateurs '='
  // pour que le binding soit explicite.
  LagrangianParam & op_Equal_1(float _value) { return this->operator=(_value); }
  LagrangianParam & op_Equal_2(const LagrangianParam &_l) { return this->operator=(_l); }
#endif

public:
    /**Constructeur par dfaut*/
    LagrangianParam():id(0),model(0) {}

    LagrangianParam(Model & _model, std::string constraintName );
    
    LagrangianParam(Model & ,VarId);

    /**Constructeur de recopie*/
    LagrangianParam(const LagrangianParam & _n):id( _n.id), model(_n.model)  {}
    
    /**Oprateur de recopie*/
    LagrangianParam & operator= (const LagrangianParam & _m)
    {
      model = _m.model;
      id= _m.id;
      return *this;
    }

    /**Geter de l'identifiant de variable
       @return L'identifiant de variable sous le format VarId.
     */
    VarId GetId() const { return id;}

    /**Geter du modle
       @return Un pointeur sur le Model o est dfinie la varaible.
     */
    Model * GetModel() const { return model;}

    float GetValue() const;
    void SetValue( float );

    /**Oprateur d'attribution =*/
    LagrangianParam & operator= (float _value)
    {
    	SetValue(_value);
    	return *this;
    }

    /**Oprateur d'attribution +=*/
    LagrangianParam & operator+= (float _value)
    {
    	SetValue(_value + GetValue());
    	return *this;
    }

    /**Oprateur d'attribution -=*/
    LagrangianParam & operator-= (float _value)
    {
    	SetValue( GetValue() - _value);
    	return *this;
    }

    //Destructeur
    ~LagrangianParam() {}
};

#ifdef COMPILE_FOR_PYTHON
	void export_LagrangianParam();
#endif

}


#endif
