/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __NUM_H_
#define __NUM_H_

#include "MuteVar.h" //pour MuteVar::VarType

namespace Modelib {

//!Classe representant les nombres de base.
/**Elle sert surtout au transtypage pour les expressions et les contraintes grce  ses constructeurs  un argument
   qui ne sont pas dfinit en explicit pour que le compilateur puisse faire des conversions implicites  la cration
   d'expression ou de contraintes. \n\n
   Elle est normalise et  3 hritires Int,Float,Bool qui servent  la facilit de dclaration.
 */

class Num
{
  protected:
  MuteVar::VarType type; //!< Type du nombre
  float value; //!< Value du nombre
  public:
    /**Constructeur par dfaut*/
    Num(MuteVar::VarType _type = MuteVar::FLOAT,float _value = 0)
    : type(_type) {*this = _value;}
    /**Constructeur  l'aide d'un entier
       @param entier Entier  attribuer  la nouvelle value
     */
    Num(int entier):type( MuteVar::INT ), value( static_cast<float>(entier) ) {}
    /**Constructeur  l'aide d'un entier non signe
       @param entier Entier  attribuer  la nouvelle value
     */
    Num(unsigned int entier):type( MuteVar::INT ), value( static_cast<float>(entier) ) {}
     /**Constructeur  l'aide d'un entier
       @param float rel  attribuer  la nouvelle value
     */
    Num(float _value):type( MuteVar::FLOAT ), value( _value ) {}
     /**Constructeur  l'aide d'un entier
       @param double rel  attribuer  la nouvelle value
     */
    Num(double _value):type( MuteVar::FLOAT), value( static_cast<float>(_value) ) {}
    /**Constructeur de recopie*/
    Num(const Num & _n):type(_n.type), value(_n.value) {}
    /**Oprateur de recopie*/
    Num & operator= (const Num & _n)
    {
      type = _n.type;
      value = _n.value;
      return *this;
    }
    /**Oprateur d'attribution de Value.*/
    Num & operator= ( float _value )
    {
      switch(type)
      {
      	  case MuteVar::FLOAT:
            value = _value;
          break;
      	  case MuteVar::INT:
            value = static_cast<int>(_value);
          break;
      	  case MuteVar::BOOL:
            value = _value;
          break;
      }
      return *this;
    };      
   
    /**Geter de la value
      @return La value de la contante convertie en flottant
      */
    float GetValue() const {return value;}

    /**Seter de la value
      @return La value de la contante convertie en flottant
      */
    void SetValue( float f) { *this = f;}
    /**Geter du type de la constante
      @return Le type de le contante sous le format MuteVar::VarType
      */
    MuteVar::VarType GetType() const { return type; }
    
    /**Seter du type de la constante
      @return Le type de le contante sous le format MuteVar::VarType
      */
    void SetType(MuteVar::VarType _type)  { type = _type; }

    ~Num() {}
    
};

///Classe reprsentant les constantes entires
class Int:public Num
{
  public:
    /**Constructeur de l'utilisateur*/
    Int(int _entier = 0):Num(MuteVar::INT, static_cast<float>(_entier)) {}
};  

///Classe reprsentant les constantes relles
class Float:public Num
{
  public:
    /**Constructeur de l'utilisateur*/
    Float(float flottant = 0.0f):Num(MuteVar::FLOAT, flottant) {}
};

///Classe reprsentant les constantes boolennes
class Bool:public Num
{
  public:
    /**Constructeur de l'utilisateur*/
    Bool(bool booleen = false):Num(MuteVar::BOOL, (booleen?1.0f:0.0f)) {}
};  
  
}

#endif
