/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __NumVarHyperMatrix_H
#define __NumVarHyperMatrix_H

#include "Constant.h"
#include "MuteVar.h"

#include <vector>
#include <string>
#include <stdarg.h>
#include <iostream>

namespace Modelib {

//Pour viter les dfinitions circulaire
class Model;

///Classe reprsentant des familles d'inconnues
/**
  Son utilit rside dans les tableaux multidimensionnels de variables. 
  Elle le mme nom  leur coordonnes dans l'hyper matrice prs.
  A n'utiliser qu'en dernier recourt
*/
template <unsigned N>
class NumVarHyperMatrix
{
  protected:
  Model * model;
  std::vector< NumVarHyperMatrix<N-1> > NumVarFamily;
  MuteVar::VarType type;
  friend class NumVarHyperMatrix<N-1>;
  std::vector<unsigned> size;


  void ConstructNumVarHyperMatrix( Model & _model,
                                MuteVar::VarType _type,
                                const std::string & name,
                                std::vector<unsigned> & pile,
                                std::vector<std::string> & format,
                                const std::string & Id = "")
  {
   	  model = &_model;
    	type = _type;
      char buf[12];
      for(unsigned i = 0; i < pile[N-1]; ++i )
      {
        sprintf(buf,"%d",i);
        NumVarFamily.push_back( NumVarHyperMatrix<N-1>(_model,_type,name,pile,format, format[N-1] + buf+Id) );
      }
   }

  public:
    /**Constructeur par dfaut */
    NumVarHyperMatrix():model(0),type(MuteVar::FLOAT) {}

    /**Constructeur standart*/
    NumVarHyperMatrix( Model & _modele,
                  MuteVar::VarType _type,
                  const std::string & name,
                  ... )
    {
      va_list pile;
    	va_start(pile,name);
      for(unsigned i = 0; i < N; ++i)
      {
        unsigned p = va_arg(pile,unsigned);
        size.push_back( p );
      }

      std::string nom;
      std::vector< std::string > format;
      unsigned i = 0;
      while( i < name.length() && name[i] != '%')
      {
      	nom += name[i];
        ++i;
      }
      if( i < name.length() )
      {
      	 std::string tmp;
         while( i < name.length() )
      	 {
      	 	  if( name[i] == '%' && (i + 1 < name.length() && name[i+1] == 'd'))
            {
            	format.push_back(tmp);
              ++i;
            }
            else
              tmp += name[i];
            ++i;
      	 }
      }
      while(format.size() < N)
      {
      	format.push_back("");
      }

      ConstructNumVarHyperMatrix(_modele,_type,nom,size,format);
    }

    NumVarHyperMatrix( Model & _model,
                  MuteVar::VarType _type,
                  const std::string & name,
                  std::vector<unsigned> & pile,
                  std::vector<std::string> & format,
                  const std::string & Id = "")
    {
       ConstructNumVarHyperMatrix( _model,_type,name,pile,format,Id);
    }

    NumVar operator()  (unsigned n, ...) const
    {
      va_list pile;
    	va_start(pile,n);
    	std::vector<unsigned> v;
      v.push_back( n );
      for(unsigned i = 0; i < N-1; ++i)
      {
        unsigned p = va_arg(pile,unsigned);
        v.push_back( p );
      }
      return NumVarFamily[v.back()](v);
    }

    NumVar operator()  ( const std::vector<unsigned> & pile) const
    {
    	return (NumVarFamily[pile[N-1]])(pile);
    }

    std::vector<unsigned> Size() const
    {
    	 return size;
    }

    unsigned Size(unsigned dimension) const
    {
    	 return size[dimension];
    }

    MuteVar::VarType GetType() const { return type;}


    ~NumVarHyperMatrix() {}
};

///Classe reprsentant des familles d'inconnues
/**
  Son utilit rside dans les tableaux multidimensionnels de variables. 
  Elle le mme nom  leur coordonnes dans l'hyper matrice prs.
  A n'utiliser qu'en dernier recourt
*/
template<>
class NumVarHyperMatrix<0>
{
   protected:
      NumVar var;
    
   
   public:
      NumVarHyperMatrix( Model & _model,
                  MuteVar::VarType _type,
                  const std::string & name,
                  std::vector<unsigned> & pile = std::vector<unsigned>(),
                  std::vector<std::string> & format,
                  const std::string & Id = "" )
     {
        var = NumVar(_model,0,Infinity,_type, name+Id );
     }

     NumVar operator()  ( const std::vector<unsigned> & pile) const
     {
    	  return var;
     }

    ~NumVarHyperMatrix() {}
};

}

#endif
