/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#ifndef __SOLUTION_H
#define __SOLUTION_H

#include <list>
#include <string>
#include <map>

namespace Modelib {

/**
	Conteneur d'une solution complte
*/
struct Solution
{
	std::list<std::string> contraintesNOM; //!< sauvegarde des noms des contraintes
	std::list<std::string> variablesNOM;   //!< sauvegarde des noms des variables
	std::list<float>       contraintesVAL; //!< sauvegarde des valeurs des contraintes
	std::list<float>       variablesVAL;   //!< saveugarde des valeurs des variables

  	std::pair<std::string, float> objective; //!< sauvegarde de la valeur et du nom de la fonction objective

	//
	// Mthodes lies  la gestion des solutions
	//

	//! Constructeurs
	Solution() {}
	Solution(const Solution&);
	//! Destructeur
	~Solution() {}

	void Clear();

	//
	// Accesseurs
	//
	inline std::list<std::string>& GetConstaintsNames()  {
		return contraintesNOM;
	}
	inline std::list<std::string>& GetVariablesNames()  {
		return variablesNOM;
	}
	inline std::list<float>& GetConstaintsValues()  {
		return contraintesVAL;
	}
	inline std::list<float>& GetVariablesValues()  {
		return variablesVAL;
	}
	inline std::string& GetObjectiveName()  {
		return objective.first;
	}
	inline float GetObjectiveValue()  {
		return objective.second;
	}

	//
	// Oprateurs utiles
	//
	Solution& operator=(const Solution&);
};

}

#endif
