/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or
	without modification, are permitted provided that the following
	conditions are met:

		* Redistributions of source code must retain the above copyright
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright
		  notice, this list of conditions and the following disclaimer in the
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names
		  of its contributors may be used to endorse or promote products
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __HTMLFILE_H
#define __HTMLFILE_H

#include "Model.h"
#include <string>
#include <fstream>
#include <stdexcept>

namespace Modelib {

/**
	Cette classe permet de generer un fichier HTML.
	Le fichier est considere siimplement comme un flux, tout est
	pris en compte pour que l'affichage HTML soit correct.
*/
class HTMLFile
{
	std::string    nom;	//!< Nom du fichier a ouvrir
	std::ofstream html;	//!< Flux vers le fichier
	bool     openLater; //!< Utilise lorsque l'on declare le fichier avant de l'utiliser vraiment

	protected:
		
		//! Construction de notre entete de fichier HTML (avec declaration du CSS)
		void BuildHeader()
		{
		
            std::string __CSS[9] = {
                " body {background: #449; font: 13px \"Trebuchet MS\" verdana; border-left:2px solid #222;border-top: 2px solid #222;border-right:  8px solid #222;border-bottom: 8px solid #222;}",
                "#container {background: #efefef; padding: 2px 10px 2px 10px;}",
                "#container h1 {background: #ccc;border: 1px solid;font: 20px \"Trebuchet MS\" verdana;padding: 0px 5px 0px 5px;}",
                "#container h2 {border-right  :15px solid #666;border-top   : 1px solid #666;border-bottom: 1px solid #666;font: 16px \"Trebuchet MS\" verdana; padding: 0px 5px 0px 5px;}",
                "#container pre {position: relative;background: #fff;border: 1px dashed #669;border-left: 3px solid #669;font: 12px Courier, Tahoma;padding: 2px 10px 2px 10px;}",
                "#container a       {color: #339; text-decoration: none;}",
                "#container a:hover {color: #000; text-decoration: none;}",
                "hr {border:1px solid;}",
                "#container h3 {background:#ddd; border:2px solid #f63;font: 20px \"Trebuchet MS\" verdana;padding: 0px 5px 0px 5px;text-align:center;}",
            };
      
            html << "<html><head><title>"<<nom<<" - Linear program modeling</title><meta http-equiv=\"Content-type\" content=\"text/html; charset=iso-8859-1\"/><style type=\"text/css\">"<<std::endl;

			for (unsigned i=0;i<9;i++)
			{
			    html << __CSS[i] << std::endl;
                html.flush();
            }

			html << "</style></head><body><div id='container'>" << std::endl;
			html.flush();
		}		

		//! Fermer correctement le fichier HTML
		void BuildFooter()
		{
			html << "</body></html>";
			html.flush();
		}

		//! Constructeur par defaut qui ne doit jamais etre appele de l'exterieur
		HTMLFile() {};

	public:
	
		/**
			Constructeur
			@param _nom			Le nom du fichier HTML
			@param _openLater	On ouvrira le fichier plutard ?
		*/
		HTMLFile(const std::string& _nom, bool _openLater=false) : nom(_nom),openLater(_openLater)
		{
			if (openLater==false)
			{
				html.open(nom.c_str());
                if( !html.is_open())
                    throw runtime_error("HTMLFile::HTMLFile() : I/O Error");
			    BuildHeader();
			}
		}
		//! Destrcteur, on ferme le fichier...
		~HTMLFile() { html.close(); }
		
		//! Ouvrir notre flux, meme si une instance de HTMLFile a ete definie avant
		void Open()
		{
			html.open(nom.c_str());
			BuildHeader();
			html.flush();
		}

		//! Fermer le fichier sans detruire notre instance de HTMLFile
		void Close()
		{
			BuildFooter();
			html.flush();
			html.close();
		}
		
		// Diverses surcharges d'operateur pour faciliter l'ecriture

		HTMLFile& operator<<(const Model& _m);  			//< using <pre> keyword for html rendering
		HTMLFile& operator<<(const std::string& _s);
		HTMLFile& operator<<(const float _f);
		HTMLFile& operator<<(const char  _c);
		HTMLFile& operator<<(const int   _i);


		//! Ecrire une nouvelle ligne dans le fichier HTML
		void NewLine()
		{
			html << "<br />" << std::endl;
			html.flush();
		}
		
		//! Hack pas beau pour ne pas s'embeter si on veut lier plusieurs flux
        std::ostream & getStream() {return html;}
};

}

#endif

