/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <boost/python.hpp>
#include <boost/python/class.hpp>
#include <boost/python/module.hpp>
#include <boost/python/def.hpp>
#include <boost/python/implicit.hpp>

namespace Modelib {

void export_Expr()
{
    using namespace boost::python;
    class_<Expr>("Expr",init<>())      // constructeurs
	  .def(init<Expr>())
      .def(init<const NumVar&>())
      .def(init<float>())
      .def(init<int>())
      .def(init<double>())
      .def(init<unsigned>())
      .def(init<const Num&>())
      .def(init<const Num&, const NumVar&>())
      .def(init<const Num&, const NumVar&, const LagrangianParam&>())
      .def("ASS", &Expr::ASS_prim,return_internal_reference<>())          // mthodes
      .def("ASS", &Expr::ASS_lag,return_internal_reference<>())
      .def("Size", &Expr::Size)
      .def("IsNum", &Expr::IsNum)
      .def("IsVar", &Expr::IsVar)
      .def("GetConstant", &Expr::GetConstant)
      //.def("__eq__", &Expr::operator=,return_internal_reference<>())                // oprateurs surchargs
      //.def(self = self)
	  .def("__iadd__", &Expr::__iadd__,return_internal_reference<>())
      .def("__isub__", &Expr::__isub__,return_internal_reference<>())
      .def("__imul__", &Expr::__imul__,return_internal_reference<>())
      .def("__str__", &Expr::ostr_1,return_internal_reference<>())
      .def("__str__", &Expr::ostr_2,return_internal_reference<>())
	 ;

  // dfinitions des fonctions construisant une expression
  // depuis des combinaisons d'entits
  def("Sum", &Sum);
  def("__mul__", &omul_1);
  def("__mul__", &omul_2);
  def("__mul__", &omul_3);
  def("__mul__", &omul_4);
  def("__mul__", &omul_5);
  def("__mul__", &omul_6);
  def("__add__", &oplus);
  def("__sub__", &ominus);
  def("Out", &Oute);
  
  // Conversion implicites pour permettre des Expr += unsigned etc.
  // Python les verra comme des Expr += Expr  
  implicitly_convertible<int,Expr>();
  implicitly_convertible<NumVar,Expr>();  
  implicitly_convertible<IntVar,Expr>();  
  implicitly_convertible<FloatVar,Expr>();  
  implicitly_convertible<float,Expr>();  
  implicitly_convertible<unsigned,Expr>();  
  implicitly_convertible<double,Expr>();  
  implicitly_convertible<Num,Expr>();
  implicitly_convertible<Int,Expr>();
  implicitly_convertible<Float,Expr>();

}

}
