#!/usr/bin/env python
"""
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or
	without modification, are permitted provided that the following
	conditions are met:
	
		* Redistributions of source code must retain the above copyright
		  notice, this list of conditions and the following disclaimer.
	
		* Redistributions in binary form must reproduce the above copyright
		  notice, this list of conditions and the following disclaimer in the
		  documentation and/or other materials provided with the distribution.
	
		* Neither the name of Quentin Lequy or Romain Gaucher nor the names
		  of its contributors may be used to endorse or promote products
		  derived from this software without specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

import sys, os, re
import getopt

NOM_EXEC   = "ModelMath"
usage = "Usage : %s arguments\n\
	\t--with_python -p   Necessite Boost.Python et Python2.2(au moins) d'installe, creation des librairies pour Python\n\
	\t--with_png    -i   Permet de faire une sortie graphique (format PNG) des matrices creuses\n\
	\t--with_latex  -l   On pourra sortir un fichier en LaTeX dans le programme\n\
	\t--with_html   -w   Sortie d'un fichier HTML (utilise CSS1)" % sys.argv [0]
with_python = 0
with_png    = 0
for_windows = 0
with_latex  = 1
with_html   = 1

opts = {}
arguments = {}
sources=[]
headers=[]

# Tout ce qu'il faut pour le makefile
# il se divise en plusieurs sections
dependances = "#------------------------------------------------------\n"
compilo="#------------------------------------------------------\n\
#Compilateur et option\n\
CC=g++\n\
AR=ar\n\
CFLAGS=-Wall -pedantic -O3 -malign-double -ffast-math -std=c++98 -Iinclude\n"
python="linking=-L/usr/local/lib -lpython2.2 -lboost_python -DBOOST_PYTHON_STATIC_MODULE /usr/local/lib/libboost_python-gcc.a\n\
chemin=-I/usr/local/include/boost-1_32 -I/usr/include/python2.2\n"
wo_python="chemin=\nlinking=\n"
entete="\
# Makefile genere par make.py\n\
# Ce logiciel a ete cree pour Modelib (ISIMA/2005)\n\
# http://neuroo.kouette.com/modelib\n\
# Quentin Lequy et Romain Gaucher\n"
machine="#------------------------------------------------------\n\
%.o : %.cpp\n\
	$(CC) -c $(CFLAGS) $*.cpp -o $@"
exec_python="$(chemin)\n\
%.so : %.o\n\
  @echo \" so file:\"$*.o\n\
  @$(CC) -shared $*.o -o $*.so $(linking)\n"
machine2="\n\n$(EXEC) : $(OBJS)\n\
	@echo \" exe file:\"$(EXEC)\n\
	@$(CC) -o $(EXEC) -g $(OBJS) $(LFLAGS)\n\
	@echo \" generate static_lib:\" $(LIBA)\n\
	@$(AR) r  $(LIBA) $(OBJS_WITHOUT_MAIN)\n\
	@echo \" generate dynamic library: \" $(LIBSO)\n\
	@$(CC) -shared -o $(LIBSO) $(OBJS_WITHOUT_MAIN)\n\
\n\
clean:\n\
	@echo \"cleaning all files but sources\"\n\
	@rm -f $(OBJS) $(EXEC)\n\
\n\
install:\n\
	@echo \" writing the path...\"\n\
	@export MODELIB=$PWD\n\
	@export PATH=$PATH:$PWD/bin:$PWD/include\n\
	@export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$MODELIB/bin\n"

def usage_error (message, exit_code) :
	print '--> ### %s ###' % message
	print '--> %s' % usage
	sys.exit (exit_code)

def parse_cmds (opts):
	global with_python, with_png, for_windows, with_html, with_latex
	option = {}
	#print '=)> %s' % opts		
	for arg in opts:
		if arg in ('-p', '--with_python'):
			with_python = 1
		elif arg in ('-i','--with_png'):
			with_png = 1
		elif arg in ('-l','--with_latex'):
			with_latex = 1
		elif arg in ('-w','--with_html'):
			with_html = 1
		elif arg in ('-h', '--help'):
			print '%s' % usage
			sys.exit(1)
		else:
			str = "option [%s] inconnu" % arg
			usage_error(str,0)
			
	if os.name == "posix" or os.name == "mac" or os.name == "riscos":
		for_windows = 0
	else:
		for_windows = 1


def get_files():
	# listes les .cpp et .h
	global headers, sources
	all = {};
	dirname = os.getcwd()
	all = [f for f in os.listdir(dirname) if os.path.isfile(os.path.join(dirname, f))]
	# print all
	# construire l'expression reguliere permettant d'avoir les .cpp et .h
	old = re.compile("old.(.+)$")
	cpp = re.compile("(.+).cpp$")
	h   = re.compile("(.+).h$|(.+).cxx$")
	"""
	for f in all:
		if cpp.match(f,re.IGNORECASE) and not old.match(f,re.IGNORECASE):
			sources.append(f)
		if h.match(f,re.IGNORECASE) and not old.match(f,re.IGNORECASE) and f != NOM_EXEC:
			headers.append(f)
	"""
	os.chdir("sources")
	sources = [f for f in os.listdir(os.curdir) if (cpp.match(f,re.IGNORECASE)) and not old.match(f,re.IGNORECASE)]
	os.chdir("../include")
	headers = [f for f in os.listdir(os.curdir) if (h.match(f, re.IGNORECASE)) and not old.match(f,re.IGNORECASE) and f != NOM_EXEC]
	os.chdir("..")

def make_configh():
	print "Construction de Config.h"
	try:
		configh = open("include/Config.h","w")
	except IOError:
		print "Impossible d'ecrire le fichier desire"
		return 0
	else:
  		configh.write("// Fichier genere automatiquement par Buildfile.\n// Modelib / Programmes lineaires pour C++ et Python\n")
  		configh.write("#ifndef __CONFIG_H\n#define __CONFIG_H\n\n")
  		configh.write("#define __no_export\n")
		if for_windows:
			configh.write("#define WINDOWS_VERSION\n")
  		if with_png:
			configh.write("#define EXPORT_SPARSE_MATRIX_PNG\n")
  		if with_python:
			configh.write("#define COMPILE_FOR_PYTHON\n")
  		if with_latex:
			configh.write("#define USE_LATEX_FORMAT\n")
		if with_html:
			configh.write("#define USE_HTML_FORMAT\n")
  
  		configh.write("\n\n#endif\n\n")
  		configh.close()
  	return 1

def make_file():
	global sources
	print "Construction du Makefile"
	nom = "Makefile"

	try:
		makefile = open(nom,"w")
	except IOError:
		print "Impossible d'ecrire le fichier desire"
		return 0
	else:
		# mettre les sources suivant les options
		makefile.write("#------------------------------------------------------\n");
		makefile.write(entete)
		makefile.write("#------------------------------------------------------\n");
		srcs = "SRCS="
		for f in sources:
			if with_latex == 0 and f == "latexFile.cpp":
				continue
			elif with_png == 0 and f == "Png.cpp":
				continue
			elif with_html == 0 and f == "htmlFile.cpp":
				continue
			elif f == "Python.cpp" and not with_python:
				continue
			else:
				srcs = srcs + "sources/" + f + " "
		srcs = srcs + "\n"
		makefile.write(srcs)
		makefile.write("OBJS=$(patsubst %.cpp, %.o, $(SRCS))\n")
		EXEC = "EXEC=bin/%s" % NOM_EXEC
		if for_windows:
		   EXEC = EXEC+".exe"
		EXEC = EXEC + "\n"
		makefile.write(EXEC)
		makefile.write("LIBA=bin/model.a\n")
		makefile.write("LIBSO=bin/libmodel.so\n")
		# mettre les .o pour la construction de la lib
		objs = ""
		objs2= ""
		for f in sources:
			if with_latex == 0 and f == "latexFile.cpp":
				continue
			elif with_png == 0 and f == "Png.cpp":
				continue
			elif with_html == 0 and f == "htmlFile.cpp":
				continue
			elif f == "main.cpp":
			  continue
			elif f == "Python.cpp" and not with_python:
			  continue
			else:
				objs = objs + "sources/" + f + " "

		objs2 = objs.replace(".cpp",".o")
		#print objs
		objs2 = "OBJS_WITHOUT_MAIN=" + objs2 + "\n"
		makefile.write(objs2)
		
		# Chemins pour python
		if with_python:
		   makefile.write(python)

  	# compilo
		makefile.write(compilo)
		LPFLAG = "LFLAGS=-Iinclude -lm"
		if with_png == 1:
			LPFLAG += " -lpng\n"
		else:
			LPFLAG = LPFLAG + " \n"
		makefile.write(LPFLAG)
		# mettre les dependances
		makefile.write(dependances)

		# Compilation des .o
		makefile.write(machine)
    
    # Compilation des .so
		if with_python:
			makefile.write(exec_python)

    # Les autres regles du makefile
		makefile.write(machine2)
		
		makefile.close()
		return 1

# main...
try :
	options,arguments = getopt.getopt ( sys.argv[1:] , 'hilpw' ,["help","with_png", "with_latex", "with_python", "with_html"] )
except getopt.GetoptError :
	usage_error ("Illegal options", 1)
for option in options:
	opts[option[0]] = option[1]
	
parse_cmds(opts)
get_files()
make_configh()
make_file()
