/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following
	conditions are met:

		* Redistributions of source code must retain the above copyright
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright
		  notice, this list of conditions and the following disclaimer in the
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "CplexSolver.h"
#include "Utils.h"
#include "System.h"

#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdexcept>

namespace Modelib {

using namespace std;

/**
	Constructeur
*/
CplexSolver::CplexSolver()
{
	nomSolver = "cplex";
}

/**
	Destructeur
*/
CplexSolver::~CplexSolver()
{
	//
}


float CplexSolver::Solve(Solution& sol, const string& nameFile)
{
	// effacer l'ancienne solution
	sol.Clear();
	tempName = ".tmp." + nameFile;
	string solName(nameFile);
	Utils::Replace(solName, ".lp", ".sol");

	// Creer le fichier temporaire
	try {
		CreerFichierTemporaire(nameFile);
	}
	catch(exception& e)	{
		cout << e.what() << endl;
	}

	// Appeler le solveur
	Resoudre(solName);

	try {
	// Interpreter les r�ultats
	Interprete(solName, sol);
	}
	catch (exception& e)
	{
		cout << e.what() << endl;
	}

	return sol.objective.second;

	return 0.0f;
}

/**
	Creer une fichier temporaire pour CPLEX
	ecrit les différentes fonctions de CPLEX
	pour creer un script execute sous CPLEX
*/
void CplexSolver::CreerFichierTemporaire(const string& nameFile)
{
	ofstream mod(tempName.c_str());
	if(!mod.is_open())
		throw runtime_error("(CplexSolver::CreerFichierTemporaire) - I/O Error pour " + tempName);

	mod << "read " << nameFile << endl;
	mod <<" opt" << endl;
	mod << "disp sol sla -" << endl;
	mod << "disp sol var -" << endl;
}


/**
	Appeler le solveur et lui specifier de mettre le resultat dans
	solName
*/
void CplexSolver::Resoudre(const string& solName)
{
	string cmd = nomSolver;

	cmd += " <";
	cmd += tempName;

	// avoir la sortie fichier
	cmd += " >";
	cmd += solName;

	// Appel syst�e
	System::call(cmd.c_str());
}


/**
	Interpreter la solution retournee

	@param solName Le nom du fichier contenant la sortie de CPLEX
	@param sol     Le conteneur de la solution
*/
void CplexSolver::Interprete(const string& solName, Solution& sol)
{
	ifstream inFile(solName.c_str());
	if(!inFile.is_open())
	    throw runtime_error("(CplexSolver::Inteprete) - I/O Error");

	string s,sprec;
	bool Debut = true;
	bool Contraintes=false,Variables=false;

	while (getline(inFile,s))
	{
		if(Debut)
		{
			isSolved = true;
			//on cherche la ligne apres la ligne de solution
			if( s.find("Solution time")  != string::npos )
			{
				string::size_type place;
				if ( sprec.find("ptimal") != string::npos)
					isOptimal = true;

				 if( ( place=sprec.find("tive"))  != string::npos)
				 {
					float _val;
					string stmp(sprec, place + 4, sprec.size() - place - 4  );
					istringstream sstr(stmp);
					sstr >> s  >> _val;
					sol.objective = make_pair("Objective", _val);
				 }

				Debut=false;
			}
			else
			{
				//on sauvegarde la ligne precedente
				sprec = s;
			}
		}
		else
		{
			if( Contraintes == true )
			{
				istringstream sstr(s);
				string  slack, nom;
				float val;
				sstr >> slack >> nom >> val;;
				//sol.contraintesVAL.push_back(val);
				//sol.contraintesNOM.push_back(nom);
			}

			if(Variables == true )
			{
				istringstream sstr(s);
				string   nom;
				float val;
				sstr >>  nom >> val;;
				sol.variablesVAL.push_back(val);
				sol.variablesNOM.push_back(nom);
			}

			if( Contraintes==false && Variables==false)
			{
				Contraintes = (s.find("Constraint")  != string::npos);
			}

			if( Contraintes==true && Variables==false)
			{
				Variables = (s.find("Variable")  != string::npos);
			}
		}
	}
	inFile.close();
}
}

