/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "LagrangianParam.h"
#include "Model.h"
#include <iostream>

namespace Modelib {

using namespace std;


/*-----------------------------------------------------------------------------------------------------------*/
/**!
   Constructeur requierant un modele pour tre utiliser. Facilite la lisibilit des dclarations
   et permet de s'assurer que le modele existe et est valide.\n
   @param _Model modle d'existence du multiplicateur de Lagrange
   @param _value value du multiplicateur de Lagrange
   @param _nom nom de la variable
  */

LagrangianParam::LagrangianParam(Model & _model, string constraintName )
{
  Constraint c = _model.GetConstraint(constraintName);
  //si la contrainte existe
  if( !(c == Constraint())  )
  {
     model = &_model;
	  
     id = c.GetId();
     //il prend l'identifiant de la variable
  }
  else
  {
  	cerr << "Warning in LagrangianParam::LagrangianParam : constraint doesn't exist" << endl;
    model = 0;
    id = 0;
    //sinon elle prend les caractristiques de la contrainte nulle
  }
}
/*-----------------------------------------------------------------------------------------------------------*/
/**!
   Constructeur requierant un modele pour tre utiliser. Facilite la lisibilit des dclarations
   et permet de s'assurer que le modele existe et est valide.\n
   @param _Model modle d'existence du multiplicateur de Lagrange
   @param _value value du multiplicateur de Lagrange
   @param _nom nom de la variable
  */

LagrangianParam::LagrangianParam(Model & _model, VarId _id )
{
  Constraint c = _model.GetConstraint(_id);
  //si la contrainte existe
  if( !(c == Constraint())  )
  {
     model = &_model;
     id = _id;
     //il prend l'identifiant de la variable
  }
  else
  {
  	cerr << "Warning in LagrangianParam::LagrangianParam : constraint doesn't exist" << endl;
    model = 0;
    id = 0;
    //sinon elle prend les caractristiques de la contrainte nulle
  }
}

/*-----------------------------------------------------------------------------------------------------------*/
/**
  Ceter de la value du multiplicateur de Lagrange .
  @return La value du multiplicateur de Lagrange
 */
float LagrangianParam::GetValue() const
{
   //si le modle existe
   if(model)
   {
   	  return model->GetLagrangianParamValue(id);
   	  //on retoune la valeur qui se trouve dans Model::constraintMap
   }
   else
   {
  	  cerr << "Warning in LagrangianParam::GetValue : undefined model" << endl;
      return 0;
      //sinon on retourne 0
   }
}

/**
  Seter de la value du multiplicateur de Lagrange .
  @param La value du multiplicateur de Lagrange
 */
void LagrangianParam::SetValue(float _value)
{
  //si le modele existe
  if(model)
     model->SetLagrangianParamValue(id,_value);
     //on appelle la fonction qui va changer la valeur du lagrangien dans Model::constraintMap
  else
  	 cerr << "Warning in LagrangianParam::SetValue : undefined model" << endl;

}


#ifdef COMPILE_FOR_PYTHON
       #include "py_LagrangianParam.cxx"
#endif

}
