/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "NumVarArray.h"
#include "Model.h"

namespace Modelib {

/*----------------------------------------------------------------------------*/

/**
  Constructeur principal\n
  Sert principalement  gnrer les noms des variables NumVar en fonction du nom de dpart
  de la famille.
*/

NumVarArray::NumVarArray( Model & _modele,
             unsigned size,
             float _lb,
             float _ub,
             MuteVar::VarType _type,
             const std::string & nom,
             const std::string & fin )
: modele(&_modele),type(_type)
{
  if(nom != "unknow")
    for(unsigned i=0;i < size; ++i)
    {
      char id[16];
      sprintf(id,"%d",i);
      varArray.push_back(new NumVar(_modele,_lb,_ub,_type,nom+id+fin));
    }
  else
  for(unsigned i=0;i < size; ++i)
    varArray.push_back(new NumVar(_modele,_lb,_ub,_type));
}

/**
  Constructeur de recopie
*/
NumVarArray::NumVarArray( const NumVarArray & nva )
:type(nva.type)
{
  for(unsigned i = 0; i < nva.varArray.size(); ++i)
    varArray.push_back(new NumVar(*nva.varArray[i]));
}

/**
    Constructeur crant un NumVarArray a partir d'un vecteur de variable
    @param nva le vecteur de NumVar
  */
NumVarArray::NumVarArray( const std::vector<NumVar> & nva )
{
  if( nva.size() )
  {
    type = nva[0].GetType();
    for(unsigned i = 0; i < nva.size(); ++i)
        varArray.push_back(new NumVar(nva[i]));
  }
}


/*----------------------------------------------------------------------------*/

/**
  operateur de copie
 */
NumVarArray & NumVarArray::operator= ( const NumVarArray & nva )
{
  type = nva.type;
  for(unsigned i = 0; i < nva.varArray.size(); ++i)
    varArray.push_back(new NumVar(*nva.varArray[i]));
  return *this;
}

/*----------------------------------------------------------------------------*/

/**
  Destructeur qui dtruit chaque pointeur sur les variables du modles
 */
NumVarArray::~NumVarArray()
{
  for(unsigned i = 0; i < Size(); ++i)
    delete varArray[i];
}


#ifdef COMPILE_FOR_PYTHON
       #include "py_NumVarArray.cxx"
#endif


}

