/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "System.h"

#include <string>
#ifndef WINDOWS_VERSION
  #include <unistd.h>
  #include <netdb.h>
  #include <fcntl.h>
  #include <wait.h>

#endif

namespace Modelib {

namespace System
{
	using namespace std;

	/**
		Permet d'effectuer un appel systme

		@param str La commande d'appel
	*/
	void call (const char *str)
	{
	#ifdef WINDOWS_VERSION
		system(str);
	#else
		int p = SubSystemCall(str);
		// attendre la fin du processus
		while (wait(&p) != -1)
			;
	#endif
	}

#ifndef WINDOWS_VERSION
	int SubSystemCall(const char *str)
	{
		int p[2];
		pipe(p);
		if (fork())
		{
			close(p[1]);
			return(p[0]);
		}
		else
		{
			close(p[0]);
			close(1);
			dup2(p[1],1);
			close(p[1]);
			system(str);
			exit(0);
		}
	}
#endif

	/**
		Effacer une fichier (pour les fichiers temporaires)
		
		@param str Le nom du fichier a effacer
	*/
	void del (const char *str)
	{
	#ifdef WINDOWS_VERSION
		string s = "del " + string(str);
	#else
		string s = "rm -f " + string(str);
	#endif
		system(s.c_str());
	}

}

}

