/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "Config.h"
#include "latexFile.h"
#include <string>
#include <iostream>

namespace Modelib {

using namespace std;

/**
	Surcharge de l'operateur vers le flux de sortie
	
	@param 	_m Le model que l'on souhaite afficher
	@return	LATEXFile& , une reference sur l'objet courant modifie
*/
LATEXFile& LATEXFile::operator<<(const Model& _m)
{
  if (beauty == false)
  {
  	latex << endl << "\\begin{lstlisting}"<<endl;
  	latex << _m;
  	latex << "\\end{lstlisting}" << endl;
	}
  latex.flush();
	return (*this);
}

/**
	Surcharge de l'operateur vers le flux de sortie
	
	@param 	_s Le std::string que l'on souhaite afficher
	@return	LATEXFile& , une reference sur l'objet courant modifie
*/
LATEXFile& LATEXFile::operator<<(const std::string& _s)
{
  latex << "\t";
  if (isInSubsection) latex << "\t";
	latex << _s << endl;
	latex.flush();
	return (*this);
}

/**
	Surcharge de l'operateur vers le flux de sortie
	
	@param 	_f Le flottant que l'on souhaite afficher
	@return	LATEXFile& , une reference sur l'objet courant modifie
*/
LATEXFile& LATEXFile::operator<<(const float _f)
{
	latex << _f;
	latex.flush();
	return (*this);
}

/**
	Surcharge de l'operateur vers le flux de sortie
	
	@param 	_c Le char que l'on souhaite afficher
	@return	LATEXFile& , une reference sur l'objet courant modifie
*/
LATEXFile& LATEXFile::operator<<(const char  _c)
{
	latex << _c;
	latex.flush();
	return (*this);
}

/**
	Surcharge de l'operateur vers le flux de sortie
	
	@param 	_i L'entier que l'on souhaite afficher
	@return	LATEXFile& , une reference sur l'objet courant modifie
*/
LATEXFile& LATEXFile::operator<<(const int   _i)
{
	latex << _i;
	latex.flush();
	return (*this);
}

#ifdef COMPILE_FOR_PYTHON
	#include "py_latexFile.cxx"
#endif


}
